-----------------------------------------------------------------------------
--This program calculates the lift and induced drag coeffincient of a wing.
--This is done with a numerical approximation. The outcome of the program is 
--only valid in the area where liftcoefficient varies linearly with the angle 
--of attack (to about 15 degrees).
--The program can calculate the liftcoefficient for higher angels of attack,
--but the real wing would have stalled, so the data is of no use.  
--By using a lot of procedures it is easy to add more options to the program
--as is indicated in the procedures.
--
--For comments and/or questions mail to: E.vankampen@student.tudelft.nl
-----------------------------------------------------------------------------
include get.e
atom wingspan,airspeed,root_cord,root_aoa,wing_area,Oswald_factor
atom lift_coefficient,induced_drag_coefficient,lift
integer wingshape,number_panels
sequence y_position,ybar_position,aoa,induced_aoa,A,cord,Ci,gamma,old_gamma
sequence delta_y
lift_coefficient=0
induced_drag_coefficient=0
wing_area=0
y_position={}
ybar_position={}
aoa={}
wingspan=prompt_number("Give the wingspan in meters:",{1,80})
airspeed=prompt_number("Give the airspeed in meters per second:",{1,330})
root_cord=prompt_number("Give the cord of the wing at the root in meters:",{0.1,25})
number_panels=prompt_number("Give the number of panels:",{25,250})
root_aoa=prompt_number("Give the angle of attack at the root of the wing in degrees:",{-10,20})
wingshape=prompt_number("Select the wingshape: 1=right-angled, 2=elliptical",{1,2})
gamma=repeat(10,number_panels)
old_gamma=repeat(0,number_panels)
root_aoa=root_aoa*3.141592654/180 --Conversion to radians
delta_y=repeat(0,number_panels)
induced_aoa=repeat(0,number_panels)
-----------------------------------------------------------------------------
--The panels are used to make a mathematical model of the wing.
--Different panel distributions can be made. This one is a standard 
--distribution into panels of equal size. Other distrubutions include 
--more panels at the tip or some kind of a kwadratic distribution.
procedure standard_paneldistribution(atom wingspan,atom number_panels)
for i=0 to number_panels do
    y_position=y_position&(-0.5*wingspan+i*wingspan/number_panels)
end for
for i=1 to number_panels do
    ybar_position=ybar_position&(y_position[i]+0.5*wingspan/number_panels)
end for
for i=1 to number_panels do
delta_y[i]=wingspan/number_panels
end for
end procedure
-----------------------------------------------------------------------------
--The angle of attack can be varied over the wingspan (to simulate real wings)
--The simplest model is that of a non varying angle of attack.
procedure non_varying_aoa(atom root_aoa,atom number_panels)
    aoa=repeat(root_aoa,number_panels)    
end procedure                         
-----------------------------------------------------------------------------
--The shape of the wing (as seen from above) can be selected to analyse different 
--wings.Given below are a right-angled (e.g. Wright Flyer) and an elliptical
--(e.g. Spitfire) wing shape. Other shapes include tapered wings (e.g. Fokker 50) 
--and any other shape you can think of.
procedure right_angled(atom number_panels,atom root_cord)
cord=repeat(root_cord,number_panels)    
end procedure

procedure elliptical(atom wingspan,sequence ybar_position,atom root_cord)
cord={}
for i=1 to length(ybar_position) do
cord=cord&(root_cord*sqrt(1-power((2*ybar_position[i]/wingspan),2)))
end for
end procedure
-----------------------------------------------------------------------------
--This procedure calculates the induced angle of attack, which is less than
--the geometric angle of attack as a consequence of generating lift. 
procedure induced_angle_of_attack(sequence A,sequence gamma,atom number_panels)
atom sum
for i=1 to number_panels do
    sum=0
    for j=1 to number_panels do
        sum+=A[i][j]*gamma[j]
    end for
    induced_aoa[i]=sum
end for
end procedure
-----------------------------------------------------------------------------
function Function_A(atom ybar_i,atom y_j_minus_1,atom y_j,atom airspeed)
    atom output
    atom pi
    pi=3.141592654
    output=(1/(4*pi*airspeed))*((1/(ybar_i-y_j_minus_1))-(1/(ybar_i-y_j)))
    return output
end function
-----------------------------------------------------------------------------
function next_generation_gamma(sequence Ci,sequence aoa,sequence A,sequence gamma,atom number_panels)
atom summation,j_is_not_i
for i=1 to number_panels do
    summation=0
    for j=1 to number_panels do
        j_is_not_i=j!=i
        summation+=j_is_not_i*A[i][j]*gamma[j]
    end for                                   
    gamma[i]=(Ci[i]*(aoa[i]-summation))/(1+Ci[i]*A[i][i])
end for
return gamma
end function
-----------------------------------------------------------------------------
--MAIN PROGRAM
-----------------------------------------------------------------------------
--Select the panel distribution you want to use. (Or let the user select one during
--execution by using a menu.)
standard_paneldistribution(wingspan,number_panels)

--Select a variation of the angle of attack. (Or let the user select one during
--execution by using a menu.) 
non_varying_aoa(root_aoa,number_panels)

A=repeat(repeat(0,number_panels),number_panels)
for i=1 to number_panels do
    for j=1 to number_panels do
        A[i][j]=Function_A(ybar_position[i],y_position[j],y_position[j+1],airspeed)
    end for
end for

--The wingshape that the user has selected is called from the procedure.
if wingshape=1 then
right_angled(number_panels,root_cord)    
end if
if wingshape=2 then
elliptical(wingspan,ybar_position,root_cord)    
end if

--Calculating the wing-area.
for i=1 to number_panels do
    wing_area+=cord[i]*delta_y[i]
end for

--Calculating the gamma's (vortex strength) of the vortices that are used in
--the model, according to the Kutta-Joukowski theorem.
Ci=0.5*airspeed*2*3.141592654*cord
puts(1,"Calculating...\n")
while old_gamma[floor(length(old_gamma/2))]-gamma[floor(length(gamma/2))]>0.00001 or old_gamma[floor(length(old_gamma/2))]-gamma[floor(length(gamma/2))]<-0.00001 do
old_gamma=gamma
gamma=next_generation_gamma(Ci,aoa,A,gamma,number_panels)
end while 

--Calculating the lift-coefficient.
for i=1 to number_panels do
lift_coefficient+=(2.0/(airspeed*wing_area))*gamma[i]*delta_y[i]
end for

--Calculating the induced angle of attack.
induced_angle_of_attack(A,gamma,number_panels)

--Calculating the induced-drag-coefficient.
for i=1 to number_panels do
induced_drag_coefficient+=(2.0/(airspeed*wing_area))*gamma[i]*induced_aoa[i]*delta_y[i]
end for

--Calculating the Oswald-factor.
--This is a measure of the efficiency of the wing. An elliptical wing has a
--theoretical value of the Oswald-factor of 1.
Oswald_factor=power(lift_coefficient,2)/(induced_drag_coefficient*3.141592654*power(wingspan,2)/wing_area)

--Calculating the lift of the wing at sea level (According to the 
--International Standard Atmosphere so density is 1.225 kg/m^3).
lift=lift_coefficient*0.5*1.225*power(airspeed,2)*wing_area

puts(1,"The lift-coefficient = ")
?lift_coefficient
puts(1,"The induced-drag-coefficient = ")
?induced_drag_coefficient
puts(1,"The Oswald factor = ")
?Oswald_factor                           
puts(1,"The total lift of the wing at sea-level (in Newtons) = ")
?lift
-----------------------------------------------------------------------------



